@@ -1,5 +1,7 @@
-var audioContext, audioSource, analyser;
-var spectrum_analyser = false;
+var audioContext, audioSource, analyser, analyserRandom, canvas_ctx;
+var requestIdAnimationFrame;
+var spectrumAnalyserActive = false;
+var spectrumAnalyserShow = false;
 
 function setAudioContextVisual() {
     audio = document.getElementById("audio_with_controls");
@@ -29,6 +31,7 @@ $(document).ready(function () {
     setInterval(updateMasterprogress, 5002);
     setInterval(updateDisplay, 5001);
 
+    const canvas = document.getElementById("canvas_master");
     const file_upload = document.getElementById("fileupload");
     const local_host_sound_route = "http://localhost:5050/sound/"
     const play_btn = document.getElementById("play_btn");
@@ -37,8 +40,6 @@ $(document).ready(function () {
     audio.volume = 0.25;
     const audio_volume_control = document.getElementById("audio_volume_controller");
 
-    var analyser_random = getRandomIntInclusive(1,2);
-
     audio.addEventListener("play", function () {
         play_btn.style.display = "none";
         pause_btn.style.display = "block";
@@ -104,9 +105,9 @@ $(document).ready(function () {
         //     audio.pause();
         //     audio.currentTime = Number.MAX_SAFE_INTEGER; // chrome bug; NaN, infinity
         // }, audio_duration);
-        if (!spectrum_analyser) {
-            visualise_audio(null);    // next version gets a second colored spec analyser in each comments table cell
-            spectrum_analyser = true;
+        if (!spectrumAnalyserActive) {
+            visualiseAudio(null);
+            spectrumAnalyserActive = true;
         }
         let track = 1;
         audio.onended = function () {
@@ -139,9 +140,9 @@ $(document).ready(function () {
                 audio.src = local_host_sound_route + player_name;
                 audio.load();
                 audio.play();
-                if (!spectrum_analyser) {
-                    visualise_audio(null);
-                    spectrum_analyser = true;
+                if (!spectrumAnalyserActive) {
+                    visualiseAudio(null);
+                    spectrumAnalyserActive = true;
                 }
             }
         });
@@ -269,6 +270,8 @@ $(document).ready(function () {
 
             if (data.result === 'activate_audio') {
                 console.log('activate_audio');
+                stopVisualiseAudio();
+
                 try {
                     analyser = audioContext.createAnalyser();
                     audioSource.connect(analyser);
@@ -285,9 +288,10 @@ $(document).ready(function () {
                 audio.load();
                 audio.play();
 
-                if (!spectrum_analyser) {
-                    visualise_audio(null);
-                    spectrum_analyser = true;
+                if (!spectrumAnalyserActive) {
+                    analyserRandom = getRandomIntInclusive(1,2);
+                    visualiseAudio(null);
+                    spectrumAnalyserActive = true;
                 }
             }
             ;
@@ -371,110 +375,152 @@ $(document).ready(function () {
     }
     ;
 
-    function visualise_audio(var_canvas) {
+    function visualiseAudio(var_canvas) {
+
         // const canvas = document.getElementById("canvas_" + var_canvas);
-        const canvas = document.getElementById("canvas_master");
         // console.log('show canvas: ' + var_canvas)
-        var canvas_ctx = canvas.getContext('2d');
-        analyser.fftSize = 128;
-        const bufferLength = analyser.frequencyBinCount;
-        var barWidth = (canvas.width / bufferLength) * 2; // const barWidth = (canvas.width/2) / bufferLength;
-        const dataArray = new Uint8Array(bufferLength);
+        canvas_ctx = canvas.getContext('2d');
+        requestIdAnimationFrame = undefined;
+
         let barHeight;
         let x = 0;
 
-        rotateVisualiser();
-        function rotateVisualiser() {
-            barWidth = (canvas.width/2) / bufferLength;
-            requestAnimationFrame(rotateVisualiser);
-            x = 0;
-            analyser.getByteFrequencyData(dataArray);
-            canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
-            for (let i = 0; i < bufferLength; i++){
-                barHeight = dataArray[i] * 0.36;
-
-                canvas_ctx.save();
-                canvas_ctx.translate(canvas.width/2, canvas.height/2);
-                canvas_ctx.rotate(i * Math.PI * 4 / bufferLength);
-                const hue = i * 5;
-                canvas_ctx.fillStyle = 'hsl(' + hue + ',100%,50%)';
-                canvas_ctx.fillRect(0, 0, barWidth, barHeight);
-                x += barWidth;
-                canvas_ctx.restore();
-            }
-        }
-        // animate();
-        function animate() {
-            x = 0;
-            analyser.fftSize = 128;
-            analyser.getByteFrequencyData(dataArray);
-            canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
-            requestAnimationFrame(animate);
-            for (let i = 0; i < bufferLength; i++){
-                barHeight = (dataArray[i] / 2) + 2;
-                canvas_ctx.fillStyle = 'rgb(219, 111, 52)';  //  #d441fc
-                canvas_ctx.fillRect(x, canvas.height - barHeight - 10, barWidth, 2);
-                const red = i * barHeight / 4;
-                const green = i * 3;
-                const blue = barHeight / 2;
-                canvas_ctx.fillStyle = 'rgb('+ red + ',' + green + ',' + blue +')';
-                canvas_ctx.fillRect(x, canvas.height - barHeight, barWidth, barHeight);
-                x += barWidth;
-            }
-        }
-
-        draw();
-        function draw() {
-            requestAnimationFrame(draw);
-
-            analyser.fftSize = 2048;
-            const bufferLength = analyser.frequencyBinCount;
-            const dataArray = new Uint8Array(bufferLength);
-            analyser.getByteTimeDomainData(dataArray);
-
-            canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
-            let darkBody = getBodyColor();
-            if (darkBody) {
-                canvas_ctx.lineWidth = 1.5;
-                canvas_ctx.strokeStyle = 'rgb(219, 111, 52)'; /*"#565454fc";*/
-            } else {
-                canvas_ctx.lineWidth = 3;
-                canvas_ctx.strokeStyle = '#565454fc';
-            }
-            canvas_ctx.beginPath();
-            var sliceWidth = canvas.width * 1.0 / bufferLength;
-            var x = 0;
-
-            for (var i = 0; i < bufferLength; i++) {
-
-                var v = dataArray[i] / 128.0;
-                var y = v * canvas.height / 2;
-
-                if (i === 0) {
-                    canvas_ctx.moveTo(x, y);
-                } else {
-                    canvas_ctx.lineTo(x, y);
-                }
-
-                x += sliceWidth;
-            }
-
-            canvas_ctx.lineTo(canvas.width, canvas.height / 2);
-            canvas_ctx.stroke();
-        }
-
-          try {
-            if (analyser_random === 1) { rotateVisualiser();}
-            if (analyser_random === 2) { draw();}
-        } catch (error) {
-            console.error(error);
-        }
+        try {
+            if (analyserRandom === 1) { rotateVisualiser();}
+            if (analyserRandom === 2) { draw();}
+            // if (analyserRandom === 3) { animate();}
+        } catch (error) {console.error(error);}
 
     }
     ;
 
+    function stopVisualiseAudio() {
+       window.cancelAnimationFrame(requestIdAnimationFrame);
+       requestIdAnimationFrame = undefined;
+       spectrumAnalyserActive = false;
+    }
+    ;
 
-});
+    // animate();
+    function animate() {
+        if (!spectrumAnalyserShow) {
+            spectrumAnalyserActive = false;
+            return;
+        }
+        x = 0;
+        analyser.fftSize = 128;
+        const bufferLength = analyser.frequencyBinCount;
+        var barWidth = (canvas.width / bufferLength) * 2; // const barWidth = (canvas.width/2) / bufferLength;
+        const dataArray = new Uint8Array(bufferLength);
+
+        analyser.getByteFrequencyData(dataArray);
+        canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
+        requestIdAnimationFrame = requestAnimationFrame(animate);
+
+        for (let i = 0; i < bufferLength; i++){
+            barHeight = (dataArray[i] / 2) + 2;
+            let darkBody = getBodyColor(); 
+            if (darkBody) {
+                canvas_ctx.fillStyle = 'red';  //  #d441fc
+                canvas_ctx.fillRect(x, canvas.height - barHeight - 3, barWidth, 18);
+                canvas_ctx.fillStyle = 'rgb(219, 111, 52)';  //  #d441fc
+                canvas_ctx.fillRect(x, canvas.height - barHeight - 6, barWidth, 2);
+                canvas_ctx.fillStyle = 'blue';
+                canvas_ctx.fillRect(x, canvas.height - barHeight, barWidth, barHeight);
+            } else {
+                canvas_ctx.fillStyle = '#bc5090';  //  #d441fc
+                canvas_ctx.fillRect(x, canvas.height - barHeight - 3, barWidth, 18);
+                canvas_ctx.fillStyle = 'black';  //  #d441fc
+                canvas_ctx.fillRect(x, canvas.height - barHeight - 6, barWidth, 2);
+                canvas_ctx.fillStyle = '#565454';
+                canvas_ctx.fillRect(x, canvas.height - barHeight, barWidth, barHeight);
+            }
+
+            x += barWidth;
+
+        }
+    };
+
+    // draw();
+    function draw() {
+        if (!spectrumAnalyserShow) {
+            spectrumAnalyserActive = false;
+            return;
+        }
+        requestIdAnimationFrame = requestAnimationFrame(draw);
+        analyser.fftSize = 2048;
+        const bufferLength = analyser.frequencyBinCount;
+        const dataArray = new Uint8Array(bufferLength);
+        analyser.getByteTimeDomainData(dataArray);
+
+        canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
+        let darkBody = getBodyColor();
+        if (darkBody) {
+            canvas_ctx.lineWidth = 1.5;
+            canvas_ctx.strokeStyle = 'rgb(219, 111, 52)';
+        } else {
+            canvas_ctx.lineWidth = 1.0;
+            canvas_ctx.strokeStyle = 'red';
+        }
+        canvas_ctx.beginPath();
+        var sliceWidth = canvas.width * 1.0 / bufferLength;
+        var x = 0;
+
+        for (var i = 0; i < bufferLength; i++) {
+            var v = dataArray[i] / 128.0;
+            var y = v * canvas.height / 2;
+
+            if (i === 0) {
+                canvas_ctx.moveTo(x, y);
+            } else {
+                canvas_ctx.lineTo(x, y);
+            }
+
+            x += sliceWidth;
+        }
+
+        canvas_ctx.lineTo(canvas.width, canvas.height / 2);
+        canvas_ctx.stroke();
+    };
+
+    // rotateVisualiser();
+    function rotateVisualiser() {
+        if (!spectrumAnalyserShow) {
+            spectrumAnalyserActive = false;
+            return;
+        }
+        let div_canvas = document.getElementById('div_canvas_master');
+        analyser.fftSize = 128;
+        const bufferLength = analyser.frequencyBinCount;
+        var barWidth = (canvas.width / bufferLength) * 2; // const barWidth = (canvas.width/2) / bufferLength;
+        const dataArray = new Uint8Array(bufferLength);
+        barWidth = (canvas.width/2) / bufferLength;
+        requestIdAnimationFrame = requestAnimationFrame(rotateVisualiser);
+
+        x = 0;
+        analyser.getByteFrequencyData(dataArray);
+        canvas_ctx.clearRect(0, 0, canvas.width, canvas.height);
+        for (let i = 0; i < bufferLength; i++){
+            barHeight = dataArray[i] * 0.36;
+            canvas_ctx.save();
+            let darkBody = getBodyColor();
+            // if (!darkBody) {
+            //     canvas.style.background = "black";
+            // } else {
+            //     canvas.style.background = "rgba(26,26,26,1)";
+            // }
+            canvas_ctx.translate(canvas.width/2, canvas.height/2);
+            canvas_ctx.rotate(i * Math.PI * 4 / bufferLength);
+            const hue = i * 5;
+            canvas_ctx.fillStyle = 'hsl(' + hue + ',100%,50%)';
+            canvas_ctx.fillRect(0, 0, barWidth, barHeight);
+            x += barWidth;
+            canvas_ctx.restore();
+        }
+    };
+    
+    
+}); /* doc.window */
 
 
 function setDarkmode() {
@@ -684,6 +730,13 @@ function cookie_toggle_show_visuals() {
             canvas.style.display = "inline-block";
             div_canvas.style.display = "inline-block";
             cookie_set_show_visuals();
+
+            var ctx = canvas.getContext("2d");
+            ctx.font = "16px Arial";
+            ctx.fillStyle = "rgb(219, 111, 52)";
+            ctx.fillText("reload page to activate analyser",10,50);
+
+
         }
         if (show_visuals === 'show_visuals') {
             analyzer_badge.textContent = "show";
@@ -716,12 +769,21 @@ function cookie_start_set_text_show_visuals() {
             analyzer_badge.textContent = "hide";
             canvas.style.display = "inline-block";
             div_canvas.style.display = "inline-block";
+            spectrumAnalyserShow = true;
+            if (!spectrumAnalyserActive) {
+                var ctx = canvas.getContext("2d");
+                ctx.font = "16px Arial";
+                ctx.fillStyle = "rgb(219, 111, 52)";
+                ctx.fillText("change radio to activate analyser",10,50);
+
+            }
 
         }
         if (show_visuals !== 'show_visuals') {
             analyzer_badge.textContent = "show";
             canvas.style.display = "none";
             div_canvas.style.display = "none";
+            spectrumAnalyserShow = false;
         }
 
 
@@ -996,3 +1058,4 @@ function getRandomIntInclusive(min, max) {
 ;
 
 
+
